@@ -113,7 +113,7 @@ typedef struct {
 } PyoutputBuffer_Object;
 
 #define PyoutputBuffer_Get(v) (((v) == Py_None) ? NULL : \
-	(((PyURI_Object *)(v))->obj))
+	(((PyoutputBuffer_Object *)(v))->obj))
 
 typedef struct {
     PyObject_HEAD
@@ -121,7 +121,7 @@ typedef struct {
 } PyinputBuffer_Object;
 
 #define PyinputBuffer_Get(v) (((v) == Py_None) ? NULL : \
-	(((PyURI_Object *)(v))->obj))
+	(((PyinputBuffer_Object *)(v))->obj))
 
 typedef struct {
     PyObject_HEAD
