@@ -455,7 +455,7 @@ xmlXIncludeParseFile(xmlXIncludeCtxtPtr ctxt, const char *URL) {
         xmlFreeDoc(pctxt->myDoc);
         pctxt->myDoc = NULL;
     }
-    if ((pctxt->myDoc != NULL) && (pctxt->dict == pctxt->myDoc->dict))
+    if ((ctxt->doc != NULL) && (pctxt->dict == ctxt->doc->dict))
         xmlDictReference(pctxt->dict);
     xmlFreeParserCtxt(pctxt);
     
@@ -1334,6 +1334,11 @@ xmlXIncludeLoadDoc(xmlXIncludeCtxtPtr ctxt, const xmlChar *url, int nr) {
 	fragment = (xmlChar *) uri->fragment;
 	uri->fragment = NULL;
     }
+    if ((ctxt->incTab != NULL) && (ctxt->incTab[nr] != NULL) &&
+        (ctxt->incTab[nr]->fragment != NULL)) {
+	if (fragment != NULL) xmlFree(fragment);
+	fragment = xmlStrdup(ctxt->incTab[nr]->fragment);
+    }
     URL = xmlSaveUri(uri);
     xmlFreeURI(uri);
     if (URL == NULL) {
